unit DND_SkyBox;


interface

uses OpenGL15, math3d, sysutils;

type
  TSkyBox = class(TObject)
  private
    SKY_MAX : Single;

    FTextures : array [0..5] of cardinal;

    FList : integer;
    HasBeenSet : Boolean;
    Position : TVector3f;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Init(DepthOfView : integer; texIds : array of cardinal);
    procedure DrawBox(pos : TVector3f);
    Procedure Clear;
    Function GetSkyPosition : TVector3f;
  end;


implementation

Uses DND_Global;

constructor TSkybox.Create;
begin
  SKY_MAX := 5000;
  FList := -1;
  HasBeenSet := false;
end;

destructor TSkybox.Destroy;
begin
  inherited Destroy;
end;

Procedure TSkyBox.Clear;
begin
  SKY_MAX := 5000;
  if FList > -1 then
  glDeleteLists(FList,1);
  FList := -1;
  HasBeenSet := false;
end;

procedure TSkybox.Init(DepthOfView : integer; texIds : array of cardinal);
var sx, sy : integer;
    i : integer;
begin

  SKY_MAX := WORLD_MAX*1.4;

  HasBeenSet := True;

  Position := SetVector(0,0,0);

    // real box
    for i := 0 to 5 do
      FTextures[i] := texIds[i];
end;

procedure TSkybox.DrawBox(pos : TVector3f);
var px,py,pz : GLFloat;
    dov : single;
begin
//Exit;
if not HasBeenSet then exit;
//LastTexID := -1;

  glEnable(GL_TEXTURE_2D);

  if FList = -1 then begin
    dov := WORLD_MAX*2;
    // left, back und top sind falsch
    px := (- dov  / 2);//+pos.x;
    py := (- dov / 2);//+pos.y;
    pz := (- dov / 2);//+pos.z;

    FList := glGenLists(1);
    glNewList(FList, GL_COMPILE);
    // Back
    glColor3f(1,1,1);

    glBindTexture(GL_TEXTURE_2D, FTextures[3]);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

    glBegin(GL_QUADS);
      glTexCoord2f(1, 0); glVertex3f(px + dov, py,           pz);
      glTexCoord2f(1, 1); glVertex3f(px + dov, py + dov, pz);
      glTexCoord2f(0, 1); glVertex3f(px,          py + dov, pz);
      glTexCoord2f(0, 0); glVertex3f(px,          py,           pz);

    glEnd;
    // Front
    glBindTexture(GL_TEXTURE_2D, FTextures[2]);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

    glBegin(GL_QUADS);
      glTexCoord2f(1, 0); glVertex3f(px,	      py,           pz + dov);
      glTexCoord2f(1, 1); glVertex3f(px,          py + dov, pz + dov);
      glTexCoord2f(0, 1); glVertex3f(px + dov, py + dov, pz + dov);
      glTexCoord2f(0, 0); glVertex3f(px + dov, py,           pz + dov);
    glEnd;
    // Bottom
    glBindTexture(GL_TEXTURE_2D, FTextures[5]);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

    glBegin(GL_QUADS);
      glTexCoord2f(1, 1); glVertex3f(px,	     py, pz);
      glTexCoord2f(1, 0); glVertex3f(px,	     py, pz + dov);
      glTexCoord2f(0, 0); glVertex3f(px + dov, py, pz + dov);
      glTexCoord2f(0, 1); glVertex3f(px + dov, py, pz);
    glEnd;
    // Top
    glBindTexture(GL_TEXTURE_2D, FTextures[4]);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

    glBegin(GL_QUADS);
      glTexCoord2f(0, 0); glVertex3f(px+dov, py + dov, pz);
      glTexCoord2f(1, 0); glVertex3f(px+dov, py + dov, pz + dov);
      glTexCoord2f(1, 1); glVertex3f(px,       py + dov, pz + dov);
      glTexCoord2f(0, 1); glVertex3f(px,       py + dov, pz);
    glEnd;
    // Left
    glBindTexture(GL_TEXTURE_2D, FTextures[1]);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

    glBegin(GL_QUADS);
      glTexCoord2f(1, 1); glVertex3f(px, py + dov, pz);
      glTexCoord2f(0, 1); glVertex3f(px, py + dov, pz + dov);
      glTexCoord2f(0, 0); glVertex3f(px, py,           pz + dov);
      glTexCoord2f(1, 0); glVertex3f(px, py,           pz);
    glEnd;

    // Right
    glBindTexture(GL_TEXTURE_2D, FTextures[0]);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

    glBegin(GL_QUADS);
      glTexCoord2f(0, 0); glVertex3f(px + dov, py,           pz);
      glTexCoord2f(1, 0); glVertex3f(px + dov, py,           pz + dov);
      glTexCoord2f(1, 1); glVertex3f(px + dov, py + dov, pz + dov);
      glTexCoord2f(0, 1); glVertex3f(px + dov, py + dov, pz);
    glEnd;
    glEndList;
  end;
  // Call the Displaylist of our Skybox
//  glLoadIdentity;
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix;
  glTranslatef(pos.X, pos.y, pos.Z);
  glCallList(FList);
  glPopMatrix;
end;

Function TSkybox.GetSkyPosition : TVector3f;
begin

Result := AddVector(Position,Camera.GetCurrentPosition);

end;

end.

