unit UnitDNDMesh;
{
The DNDMesh is a basic 3d model type, the idea is to provide a simple format
which has no animation. I.E Footballs, Bowling Pins, etc.
}

interface

Uses Math3d;

Type
TDNDHeader = Record
Key : String[4]; // DND1
Version : Integer; // 1
NumOfMesh : Integer;
Max,Min : TVector3f;
end;

TMeshHeader = Record
NumOfVerts : Integer;
Texture : String[50];
end;

TMesh = Record
Header        : TMeshHeader;
Verts         : Array Of TVector3f;
TextureCoords : Array Of TVector2f;
TextureId     : Cardinal;
end;

TDNDMesh = Record
Header : TDNDHeader;
Meshs : Array Of TMesh;
end;

Function LoadDNDMesh(Filename : String; Var DNDMesh : TDNDMesh) : Boolean;
Procedure SaveDNDMesh(Filename : String; DNDMesh : TDNDMesh);

Procedure RenderDNDMesh(DNDMesh : TDNDMesh);

implementation

Uses SysUtils,Opengl15,textures;

Function LoadDNDMesh(Filename : String; Var DNDMesh : TDNDMesh) : Boolean;
Var
F : File;
X : Integer;
begin
Result := False;

  AssignFile(F, Filename);
  Reset(F,1);

  // read header
  BlockRead(F, DNDMesh.Header, Sizeof(TDNDHeader));

  If (DNDMesh.Header.Key <> 'DND1') or (DNDMesh.Header.Version <> 1) then
  exit;

  SetLength(DNDMesh.Meshs,DNDMesh.Header.NumOfMesh);

  for x := 0 to DNDMesh.Header.NumOfMesh-1 do
  begin
  BlockRead(F, DNDMesh.Meshs[x].Header, Sizeof(TMeshHeader));

  SetLength(DNDMesh.Meshs[x].Verts,DNDMesh.Meshs[x].Header.NumOfVerts);
  SetLength(DNDMesh.Meshs[x].TextureCoords,DNDMesh.Meshs[x].Header.NumOfVerts);

  BlockRead(F, DNDMesh.Meshs[x].Verts[0], DNDMesh.Meshs[x].Header.NumOfVerts * Sizeof(TVector3f));
  BlockRead(F, DNDMesh.Meshs[x].TextureCoords[0], DNDMesh.Meshs[x].Header.NumOfVerts * Sizeof(TVector2f));

    If FileExists(ExtractFileDir(ParamStr(0)) + '\' + DNDMesh.Meshs[x].Header.Texture + '.jpg') then
  LoadTexture(ExtractFileDir(ParamStr(0)) + '\' + DNDMesh.Meshs[x].Header.Texture + '.jpg',DNDMesh.Meshs[x].TextureId,False,False,False)
  else
    If FileExists(ExtractFileDir(ParamStr(0)) + '\' + DNDMesh.Meshs[x].Header.Texture + '.tga') then
  LoadTexture(ExtractFileDir(ParamStr(0)) + '\' + DNDMesh.Meshs[x].Header.Texture + '.tga',DNDMesh.Meshs[x].TextureId,False,False,False);
  end;

  CloseFile(F);
Result := True;
end;

Procedure SaveDNDMesh(Filename : String; DNDMesh : TDNDMesh);
Var
F : File;
X : Integer;
begin
  AssignFile(F, Filename);
  ReWrite(F,1);

  DNDMesh.Header.Key := 'DND1';
  DNDMesh.Header.Version := 1;

  // read header
  BlockWrite(F, DNDMesh.Header, Sizeof(TDNDHeader));

  for x := 0 to DNDMesh.Header.NumOfMesh-1 do
  begin
  BlockWrite(F, DNDMesh.Meshs[x].Header, Sizeof(TMeshHeader));

  BlockWrite(F, DNDMesh.Meshs[x].Verts[0], DNDMesh.Meshs[x].Header.NumOfVerts * Sizeof(TVector3f));
  BlockWrite(F, DNDMesh.Meshs[x].TextureCoords[0], DNDMesh.Meshs[x].Header.NumOfVerts * Sizeof(TVector2f));
  end;

  CloseFile(F);
end;

Procedure RenderDNDMesh(DNDMesh : TDNDMesh);
var
x : integer;
begin



for x := 0 to DNDMesh.Header.NumOfMesh-1 do
begin
  glBindTexture(GL_TEXTURE_2D,DNDMesh.Meshs[x].TextureId);

  glVertexPointer(3, GL_FLOAT, sizeof(TVector3f), @DNDMesh.Meshs[x].Verts[0].X);
  glTexCoordPointer(2, GL_FLOAT, sizeof(TVector2f), @DNDMesh.Meshs[x].TextureCoords[0].X);

  glDrawArrays(GL_TRIANGLES, 0, DNDMesh.Meshs[x].Header.NumOfVerts);
end;

end;

end.
